@@ -54,89 +54,6 @@ module Opt = struct
     | Some _ -> false
 end
 
-module IntMap = Map.Make(
-  struct
-    type t = int
-    let compare = compare
-  end)
-
-module Environment : 
-sig
-  type t
-
-  val empty : t
-  val new_variable : ?eq:('a -> 'a -> bool) -> 'a -> (t -> 'a -> t) * (t -> 'a)
-
-  val equal : t -> t -> bool
-
-end =
-struct
-
-  type t = (bool -> bool) IntMap.t
-    (* Those functions [f] hide a value in their closure.
-       It can be accessed thanks to their side effet.
-       [f true] is true iff the last function called, having the same
-       identifier, holded the same value as [f].
-       [f false] is true iff f hold the default value. *)
-
-  let empty = IntMap.empty
-
-  let new_id = 
-    let id = ref 0 in
-    fun () -> incr id; !id
-
-  let new_variable ?(eq=(=)) default =
-    let id = new_id () in
-    (* reference shared by all variable with the same id *)
-    let v = ref default in
-    (* add in the map [t] a function holding data [x] *)
-    let set t x =
-      let f test_x_equal_default =
-	let old = !v in
-	v := x;
-	if test_x_equal_default then eq x default else eq x old
-      in
-      IntMap.add id f t
-    in
-    (* recover the hidden value *)
-    let get t =
-      ( try ignore ((IntMap.find id t) false) with Not_found -> v := default );
-      !v
-    in
-    set,get
-
-  let equal t1 t2 =
-    (* fold returns keys in ascending order so to_list reverse to
-       descending order *)
-    let to_list t = IntMap.fold (fun k x l -> (k,x)::l) t [] in
-    let l1 = to_list t1 in
-    let l2 = to_list t2 in
-    let rec check l1 l2 =
-      match l1,l2 with
-	| [], [] -> true
-	| (k1,f1)::q1,(k2,f2)::q2 when k1 = k2 ->
-	  let _ = f1 false in
-	  (* now the shared reference [v] holds the value of f1 *)
-	  let eq = f2 false in
-	  (* f2 checks equality against the shared reference *)
-	  if eq then check q1 q2 else false
-	| (k1,f1)::q1,(k2,f2)::q2 when k1 > k2 ->
-	  (* keys are in descending order, so k1 > k2 means that l1
-	     has a key that l2 doesn't have *)
-	  let eq = f1 true in
-	  if eq then check q1 ((k2,f2)::q2) else false
-	| (_,f1)::q1,[] ->
-	  let eq = f1 true in
-	  if eq then check q1 [] else false
-	| l1,l2 ->
-	  check l2 l1
-    in
-    check l1 l2
-
-end
-
-type env = Environment.t
-type position = { pos_name : string; pos_id : int }
 
 type mode = M | T | A
 type arg_kind = Bracket | Brace | NoBr
@@ -144,173 +61,124 @@ let bracket = Bracket
 let brace = Brace
 let nobr = NoBr
 
-type elt =
-  | Command of string * (mode * arg_kind * t) list * mode
+type 'r elt =
+  | Command of string * (mode * arg_kind * 'r) list * mode
   | Text of string
-  | Environment of string * (mode * t) option * (mode * t) list *
-      (mode * t) * mode
-  | Mode of mode * t
-  | Set of ( env -> env )
-  (* update the environment in the remaining ast, replaced by [Concat
-     []] in fixpoint *)
-  | Get of ( env -> t )
-  (* fixpoint apply the function to the current environment to obtain
-     [t], then replace this node by [t] then
-     traverse [t]. *)
-  | Get_position of ( env -> t ) * position
-  (* like Get, but the environment is taken at the given position *)
-  | Place of position
-  (* fixpoint store the current environment under the name [position]
-     and replace this node by [Concat []] *)
-and t = elt Clist.t
+  | Environment of string * (mode * 'r) option * (mode * 'r) list *
+      (mode * 'r) * mode
+  | Mode of mode * 'r
 
-let concat l = Clist.list_concat l
-let empty = Clist.empty
+type t = X of (t elt,t) Variable.with_var Clist.t
+type raw = R of raw elt Clist.t
+
+let unX = function X l -> l
+
+let concat l = X (Clist.list_concat (List.map unX l))
+let empty = X (Clist.empty)
+let of_elt e = X (Clist.singleton e)
 
 (******************************************************************************)
 (*                                  Variables                                 *)
 (******************************************************************************)
 
-type 'a variable =
-    { set : Environment.t -> 'a -> Environment.t;
-      get : Environment.t -> 'a;
-      equal : 'a -> 'a -> bool;
-      var_name : string;
-      var_printer : 'a -> string }
+type 'a variable = 'a Variable.t
+type position = Variable.position
+type env = Variable.env
 
-let variable ?(eq=(=)) ?(name="unnamed") ?(printer=fun _ -> "undefined printer" ) x =
-  let set,get = Environment.new_variable ~eq x in
-  { set = set; get = get; equal = eq; var_name = name; var_printer = printer }
+let variable = Variable.make
 
-let position : ?name:string -> unit -> position =
-  let id = ref 0 in
-  fun ?name () -> incr id;
-    let name =
-      match name with
-	| None -> "unnamed position " ^ (string_of_int !id)
-	| Some n -> n
-    in
-    { pos_name = name ; pos_id = !id }
+let position = Variable.position
 
-let get_in_env v env = v.get env
+let get_in_env = Variable.get_in_env
 
-exception Multiple_place of position
-exception Fixpoint_divergent of position list list
-
-
-let fixpoint ?(env=Environment.empty) ?(iterations=10) t =
-  (* TODO: controler le nombre d'iterations par une option de melt
-     et mettre une fonction 'set number of iterations' *)
-  let placed_tbl = Hashtbl.create 0 in
-  (* table of already placed positions *)
-  let position_tbl = Hashtbl.create 0 in
-  (* position -> env *)
-  let get_position_env pos =
-    try Hashtbl.find position_tbl pos.pos_id
-    with
-      | Not_found -> Environment.empty
-  in
-  let assert_not_placed i =
-    if Hashtbl.mem placed_tbl i
-    then raise ( Multiple_place i )
-    else Hashtbl.add placed_tbl i ()
-  in
-  let positions_changed = ref [] in
-  (* list of position that changed during the current/last application of iter*)
-  let changelog = ref [] in
-  (* list of list of position that changed during each iteration *)
-  let rec iter_elt env = function (* go througt the ast updating the environment and
-				 appliying Get functions *)
+(* spiwack: this function is mostly implied by its types.
+   we could lower the apparent complexity by having a few
+   combinators on the state monad. *)
+let compute_elt : ('e->'r->'e*'k) -> 'e -> 'r elt -> 'e*'k elt
+  = fun m env -> function
     | Command (s, l , mode) ->
-      let l,env =
-	let aux (l,env) (mode,kind,t) =
-	  let (t,env) = iter env t in
-	  ((mode,kind,t)::l,env)
+      let env,l =
+	let aux (env,l) (mode,kind,t) =
+	  let (env,t) = m env t in
+	  (env,(mode,kind,t)::l)
 	in
-	List.fold_left aux ([],env) l
+	List.fold_left aux (env,[]) l
       in
-      Clist.singleton (Command (s, List.rev l, mode)),env
+      env ,
+      Command (s, List.rev l, mode)
     | Mode (mode, t) ->
-      let t,env = iter env t in
-      Clist.singleton (Mode (mode, t)),env
+      let env,t = m env t in
+      env , Mode (mode, t)
     | Environment (s , opt, l, (mode, t), mode') ->
       (* l'ordre d'évaluation est gratuit, je ne sais pas si c'est le
 	 bon *)
-      let opt,env =
+      let env , opt =
 	match opt with
-	  | None -> opt,env
+	  | None -> env,None
 	  | Some (mode,t) ->
-	    let t,env = iter env t in
-	    Some (mode,t),env
+	    let env,t = m env t in
+	    env,Some (mode,t)
       in
-      let l,env =
-	let aux (l,env) (mode,t) =
-	  let (t,env) = iter env t in
-	  ((mode,t)::l,env)
+      let env,l =
+	let aux (env,l) (mode,t) =
+	  let (env,t) = m env t in
+	  (env,(mode,t)::l)
 	in
-	List.fold_left aux ([],env) l
+	List.fold_left aux (env,[]) l
       in
-      let t,env = iter env t in
-      Clist.singleton begin
-	Environment (s, opt, List.rev l, (mode, t), mode' )
-      end,
-      env
-    | Text s -> (Clist.singleton (Text s),env)
-    | Set setter ->
-      empty,(setter env)
-    | Get maker ->
-      iter env (maker env)
-    | Get_position (maker,position) ->
-      let t = maker (get_position_env position) in
-      iter env t
-    | Place position ->
-      assert_not_placed position;
-      ( if Environment.equal (get_position_env position) env
-	then ()
-	else ( Hashtbl.replace position_tbl position.pos_id env;
-	       positions_changed := position :: !positions_changed ));
-      (* TODO récupérer les variables qui ont changé *)
-      empty,env
-  and iter env x =
-    Clist.fold_left begin fun (x,env) t ->
-      let (t,env) = iter_elt env t in (Clist.app x t,env)
-    end (Clist.empty,env) x
-  in
-  let rec apply n =
-    if n = 0 then raise (Fixpoint_divergent !changelog)
+      let env,t = m env t in
+      env ,
+      Environment (s, opt, List.rev l, (mode, t), mode' )
+    | Text s -> env , Text s
+
+let rec compute_clist : ('e->'a->'e*'k Clist.t) -> 'e -> 'a Clist.t -> 'e*'k Clist.t
+  = fun m env l ->
+    if Clist.is_empty l then
+      env, Clist.empty
     else
-      let (t,env) = iter env t in
-      match !positions_changed with
-	| [] -> t,env
-	| l -> ( positions_changed := [];
-		changelog := l :: !changelog;
-		Hashtbl.clear placed_tbl;
-		apply (n-1) )
-  in
-  apply iterations
+      let (env,x) = m env (Clist.head l) in
+      let (env,q) = compute_clist m env (Clist.tail l) in
+      env , Clist.app x q
+
+let rec compute : Variable.env -> t -> (Variable.env*raw)
+  = fun env x ->
+    let (env,l) = compute_clist2 env x in
+    env , R l
+and compute_clist2 : Variable.env -> t -> (Variable.env*raw elt Clist.t)
+  = fun env x ->
+  compute_clist begin fun (env:Variable.env) (xelt:(t elt,t) Variable.with_var) ->
+    Variable.compute
+      Clist.empty
+      compute_of_elt
+      compute_clist2
+      env
+      xelt
+  end env (unX x)
+and compute_of_elt : Variable.env -> t elt -> (Variable.env*raw elt Clist.t)
+  = fun env elt ->
+    let (env,elt) = compute_elt compute env elt in
+    (env,Clist.singleton elt)
+
+let fixpoint ?env ?iterations x =
+  Variable.fixpoint compute ?env ?iterations x
 
 let setf var f =
-  Clist.singleton (Set (fun env -> var.set env (f (var.get env))))
+  of_elt (Variable.setf var f)
 
 let setf2 var1 var2 f =
-  Clist.singleton (
-    Set (fun env -> var2.set env (f (var1.get env) (var2.get env)))
-  )
+  of_elt (Variable.setf2 var1 var2 f)
 
 let get ?position var f =
-  Clist.singleton begin match position with
-    | None -> Get (fun env -> f (var.get env))
-    | Some p -> Get_position ((fun env -> f (var.get env)),p)
-  end
+  of_elt (Variable.get ?position var f)
 
 let place position =
-  Clist.singleton (Place position)
+  of_elt (Variable.place position)
 
-let set x v = setf x (fun _ -> v)
+let set var x = of_elt (Variable.set var x)
 let incr_var x = setf x (fun x -> x + 1)
 let decr_var x = setf x (fun x -> x - 1)
 
-let text s = Clist.singleton (Text s)
+let text s = of_elt (Variable.raw (Text s))
 
 let vari x = get x (fun v -> text (string_of_int v))
 let varf x = get x (fun v -> text (string_of_float v))
@@ -361,7 +229,7 @@ let require_package_string acc (a, b) =
   require_package acc (text a, text b)
 
 let unusual_command ?(packages = []) name args mode =
-  let thecommand = Clist.singleton (Command (name, args, mode)) in
+  let thecommand = of_elt (Variable.raw (Command (name, args, mode))) in
   List.fold_left require_package_string thecommand packages
 
 let command ?(packages=[]) name ?opt args mode =
@@ -372,15 +240,15 @@ let command ?(packages=[]) name ?opt args mode =
 
 let environment ?(packages = []) name ?opt ?(args = []) body mode =
   let theenvironment =
-    Clist.singleton (Environment(name, opt, args, body, mode))
+    of_elt (Variable.raw (Environment(name, opt, args, body, mode)))
   in
   List.fold_left
     require_package_string
     theenvironment
     packages
 
-let (^^) x y = Clist.app x y
-let mode mode x = Clist.singleton (Mode(mode, x))
+let (^^) x y = X (Clist.app (unX x) (unX y))
+let mode mode x = of_elt (Variable.raw (Mode(mode, x)))
 
 let latex = command "LaTeX" [] T
 
@@ -432,16 +300,6 @@ end = struct
   let bol x =
     if not x.bol then newline x
 
-  let rec string_split acc from c s =
-    let index = try String.index_from s from c with Not_found -> -1 in
-    if index = -1 then
-      let sub = String.sub s from (String.length s - from) in
-      List.rev (sub::acc)
-    else
-      let sub = String.sub s from (index-from) in
-      string_split (sub::acc) (index+1) c s
-  let string_split x = string_split [] 0 x
-
   let output_string x s =
     if String.length s <> 0 then begin
       Buffer.add_string x.buf s;
@@ -458,14 +316,6 @@ end = struct
     with Not_found -> () end;
     x.line_empty <- x.line_empty && !empty
 
-  let output_string x s =
-    match string_split '\n' s with
-      | [] -> ()
-      | [s] -> output_string x s
-      | s::rem ->
-          output_string x s;
-          List.iter (fun s -> bol x; output_string x s) rem
-
   let make buf = {
     buf = buf;
     bol = true;
@@ -480,8 +330,13 @@ end = struct
     x.bol <- false
 
   let string x s =
-    force_alinea x;
-    output_string x s
+    let s = Str.full_split (Str.regexp"\n\\|\r") s in
+    List.iter begin function
+      | Str.Delim "\n" (* -> newline ~force:true x*)
+      | Str.Delim "\r" -> newline (*~force:false*) x
+      | Str.Delim _ -> assert false
+      | Str.Text l -> force_alinea x;output_string x l
+    end s
 
   let char x c =
     force_alinea x;
@@ -506,9 +361,15 @@ let ensure_mode pp from_mode to_mode f = match from_mode, to_mode with
   | M, T -> Pp.char pp '$'; f (); Pp.char pp '$'
   | T, M -> Pp.string pp "\\mbox{"; f (); Pp.char pp '}'
 
+
+(* [par] is defined here because it has a special use in the
+   prelude. *)
+let partext = "\r%\n\\par\n%\n"
+let par = text partext
+
 (* bol: "beginning of line" *)
 let rec out_elt toplevel mode pp = function
-  | Text "\\par " when toplevel ->
+  | Text x when toplevel && x = partext ->
       Pp.bol pp;
       Pp.newline ~force: true pp
   | Command(name, args, rm) ->
@@ -542,11 +403,7 @@ let rec out_elt toplevel mode pp = function
       Pp.string pp s
   | Mode(m, x) ->
       ensure_mode pp m mode (fun () -> out false m pp x)
-  | Set _
-  | Get _
-  | Get_position _
-  | Place _ -> assert false
-and out toplevel mode pp x =
+and out toplevel mode pp (R x) =
   Clist.iter (out_elt toplevel mode pp) x
 
 and command_argument pp (mode, x) before after =
@@ -568,7 +425,7 @@ and out_args =
   List.iter (out_arg pp) args
 
 let to_buffer ?(mode = T) ?env buf x =
-  let x,env = (fixpoint ?env (concat [x;place finalpos])) in
+  let env,x = (fixpoint ?env (concat [x;place finalpos])) in
   out true mode (Pp.make buf) x;
   env
 
@@ -668,17 +525,15 @@ let latex_of_size size = text (string_of_size size)
 let latex_of_int x = text (string_of_int x)
 let latex_of_float x = text (string_of_float x)
 
-let rec is_empty_elt = function
-  | Text "" -> true
-  | Mode(_, y) -> is_empty y
-  | Command _ | Environment _ | Text _
-  | Set _ | Get _ | Get_position _ | Place _ ->
-      (* You would assume that these should disappear after [compute_variables]
-         is called. However, when building the AST, this is not actually
-         the case, yet. *)
+let rec is_empty_elt x = match Variable.content x with
+  | Some (Text "") -> true
+  | Some (Mode(_, y)) -> is_empty y
+  | Some (Command _ | Environment _ | Text _)
+  | None ->
+      (* When the AST is built, we are working with variables. *)
       false
 and is_empty x =
-  Clist.forall is_empty_elt x
+  Clist.forall is_empty_elt (unX x)
 
 let none_if_empty x =
   if is_empty x then None else Some x
@@ -689,8 +544,6 @@ let optcmd name = function
 
 let labelo l = optcmd "label" (Opt.map text l)
 
-let par = text "\\par "
-
 (*******************************************************************************)
 
 type label = string
@@ -804,21 +657,27 @@ let index_exponent x y z =
 let part ?label title =
   command "part" [T, title] T ^^ labelo label
 
-let chapter ?label title =
-  command "chapter" [T, title] T ^^ labelo label
+let section_command name numbered ?label ?short title =
+  let opt =
+    match short with
+      | Some short -> Some (T, short)
+      | None -> None
+  in
+  command (if numbered then name else name ^ "*") ?opt [T, title] T ^^ labelo label
 
-let section ?label title =
-  command "section" [T, title] T ^^ labelo label
-
-let subsection ?label title =
-  command "subsection" [T, title] T ^^ labelo label
-
-let subsubsection ?label title =
-  command "subsubsection" [T, title] T ^^ labelo label
+let chapter = section_command "chapter" true
+let section = section_command "section" true
+let subsection = section_command "subsection" true
+let subsubsection = section_command "subsubsection" true
 
 let paragraph title =
   command "paragraph" [T, title] T
 
+let chapter' = section_command "chapter" false
+let section' = section_command "section" false
+let subsection' = section_command "subsection" false
+let subsubsection' = section_command "subsubsection" false
+
 let displaymath x =
   environment "displaymath" (M, x) T
 
@@ -864,11 +723,15 @@ let listoftables = command "listoftables" [] T
 type alignment = [ `L | `C | `R ]
 type array_column =  [ alignment | `Vert | `Sep of t]
 
-type array_line = {
+type array_line_normal = {
   al_columns: (int*[alignment | `I]*t) list;
   al_sep: size option;
 }
 
+type array_line =
+  | ArrayNormal of array_line_normal
+  | ArrayCommand of t
+
 let array_line ?sep ?layout xs = 
   let xs =
     match layout with
@@ -879,10 +742,12 @@ let array_line ?sep ?layout xs =
 	  error "array_line: %d columns but layout only supports %d."
             (List.length xs) (List.length l)
   in
-  { al_columns = xs;
+  ArrayNormal { al_columns = xs;
     al_sep = sep 
   }
 
+let array_command x = ArrayCommand x
+
 let array_line_width al =
   List.fold_left (fun acc (w,_,_) -> acc+w) 0 al.al_columns
 let array_line_mapi f al =
@@ -896,7 +761,7 @@ let array_line_mapi f al =
 let newline = text "\\\\\n"
 let newline_size x = text (Printf.sprintf "\\\\[%s]\n" (string_of_size x))
 
-let newpage = command "newpage" [] T
+let newpage = text"\r%\n%\n\\newpage\n%\n%\n"
 let clearpage = command "clearpage" [] T
 
 let newlinegen = function
@@ -995,9 +860,6 @@ let subfloat ?label ?caption body =
   let body = labelo label ^^ body in
   environment ?opt ~packages: [ "subfig", "" ] "subfloat" (T, body) T
 
-let minipage size x =
-  environment ~args: [A, latex_of_size size] "minipage" (T, x) T
-
 let center x = environment "center" (T, x) T
 let flushleft x = environment "flushleft" (T,x) T
 let flushright x = environment "flushright" (T,x) T
@@ -1009,8 +871,17 @@ let latex_of_array_column = function
   | `Vert -> text "|"
   | `Sep t -> concat [ text "@{" ; t ; text "}"] 
 let multicolumn w a x =
-  command "multicolumn" [(A,latex_of_int w); (A,latex_of_array_column a); (A,x)] A
-let array c l =
+  command "multicolumn" [(A,latex_of_int w); (A,latex_of_array_column a); (M,x)] A
+
+
+
+type valignment = [ `T | `C | `B ]
+let latex_of_valignment = function
+  | `T -> text "t"
+  | `C -> text "c"
+  | `B -> text "b"
+
+let array ?valign c l =
   let cols = concat begin List.map latex_of_array_column c end in
   let alignments = 
     Array.of_list (List.filter (function #alignment -> true | _ -> false) c)
@@ -1026,14 +897,20 @@ let array c l =
     | #array_column as a -> multicolumn w a x
   in
   let lines = List.map begin fun al ->
-    let width = array_line_width al in
-    if width <> numcols then
-      error "array: line with %d columns instead of %d" width numcols;
-    let lc = array_line_mapi multicolumn al in
-    concat (list_insert (text " & ") lc) ^^ newlinegen al.al_sep
+    match al with
+    | ArrayNormal al ->
+	begin
+	  let width = array_line_width al in
+	  if width <> numcols then
+	    error "array: line with %d columns instead of %d" width numcols;
+	  let lc = array_line_mapi multicolumn al in
+	  concat (list_insert (text " & ") lc) ^^ newlinegen al.al_sep
+	end
+    | ArrayCommand x -> x ^^ text"\n"
   end l in
   let body = concat lines (*(list_insert newline lines)*) in
-  environment "array" ~args: [M, cols] (M, body) M
+  let opt = Opt.map (fun a -> A,latex_of_valignment a) valign in
+  environment "array" ?opt ~args: [M, cols] (M, body) M
 
 let list_env l name = 
 (*  let items = List.map ((^^) (command "item" [] T)) l in*)
@@ -1050,6 +927,8 @@ let enumerate l = list_env l "enumerate"
 let vspace s = command "vspace" [T, latex_of_size s] T
 let hspace s = command "hspace" [T, latex_of_size s] T
 let addvspace s = command "addvspace" [T, latex_of_size s] T
+let ignorespaces = text "\\ignorespaces "
+  (* ignorespaces must not have any braces, so we use the raw [text] function *)
 let smallskip = command "smallskip" [] T
 let medskip = command "medskip" [] T
 let bigskip = command "bigskip" [] T
@@ -1059,7 +938,19 @@ let phantom x = command "phantom" [T, x] T
 let vphantom x = command "vphantom" [T, x] T
 let hphantom x = command "hphantom" [T, x] T
 
-let parbox x y = command "parbox" [A, latex_of_size x; T, y] T
+let rule_ ?lift width height =
+  let width = latex_of_size width in
+  let height = latex_of_size height in
+  let opt = Opt.map (fun l->A,latex_of_size l) lift in
+  command "rule" ?opt [A,width;A,height] A
+
+let parbox x ?valign y =
+  let opt = Opt.map (fun v -> A,latex_of_valignment v) valign in
+  command "parbox" ?opt [A, latex_of_size x; T, y] T
+let minipage size ?valign x =
+  let opt = Opt.map (fun v -> A,latex_of_valignment v) valign in
+  let args = [A,latex_of_size size] in
+  environment "minipage" ?opt ~args (T,x) T
 
 type halignment = [ `C | `L | `R | `S ]
 let latex_of_halignment = function
@@ -1067,14 +958,48 @@ let latex_of_halignment = function
   | `L -> text "l"
   | `R -> text "r"
   | `S -> text "s"
+
+type xsize = [
+    size
+  | `Width of float
+  | `Height of float
+  | `Depth of float
+  | `Totalheight of float
+  ]
+(** Horizontal box commands ({!makebox}, {!framebox} and {!raisebox})
+    can use extra size information in their definition. These are computed
+    from their content:
+    [`Width] is the width of the content
+    [`Height] is the height above the baseline
+    [`Depth] is the height below the baseline
+    [`Totalheight] is the sum of [`Height] and [`Depth]
+ *)
+
+let string_of_xsize = function
+  | `Width x -> sprintf "%f\\width" x
+  | `Height x -> sprintf "%f\\height" x
+  | `Depth x -> sprintf "%f\\depth" x
+  | `Totalheight x -> sprintf "%f\\totalheight" x
+  | #size as s -> string_of_size s
+
+let latex_of_xsize s = text(string_of_xsize s)
+
 let makeframebox name size ?halign t =
-  let size = (A, bracket,latex_of_size size) in
+  let size = (A, bracket,latex_of_xsize size) in
   let halign = Opt.map (fun h -> (A,bracket,latex_of_halignment h)) halign in
   let t = (T,brace,t) in
   unusual_command name (size::(Opt.cons halign [t])) T
 let makebox = makeframebox "makebox"
 let framebox = makeframebox "framebox"
 
+let raisebox ~shift ?fakeheight x =
+  let shift = (A,brace,latex_of_xsize shift) in
+  let opt s = (A,bracket,latex_of_xsize s) in
+  let fakeh = Opt.map (fun (h,_) -> opt h) fakeheight in
+  let faked = Opt.map (fun (_,d) -> opt d) fakeheight in
+  let x = (T,brace,x) in
+  unusual_command "raisebox" (shift::(Opt.cons fakeh (Opt.cons faked [x]))) T
+
 let noindent = command "noindent" [] T
 
 let stackrel x y = command "stackrel" [M, x; M, y] M
@@ -1167,6 +1092,7 @@ let tilde x = command "tilde" [M, x] M
 let ddot x = command "ddot" [M, x] M
 let widehat x = command "widehat" [M, x] M
 let widetilde x = command "widetilde" [M, x] M
+let overline x = command "overline" [M, x] M
 
 (*******************************************************************************)
 
@@ -1408,6 +1334,8 @@ let cmd_stmaryrd = command ~packages: ["stmaryrd", ""]
 
 let llbracket = cmd_stmaryrd "llbracket" [] M
 let rrbracket = cmd_stmaryrd "rrbracket" [] M
+let llparenthesis = cmd_stmaryrd "llparenthesis" [] M
+let rrparenthesis = cmd_stmaryrd "rrparenthesis" [] M
 
 (*******************************************************************************)
 
