@@ -390,11 +390,21 @@ val maketitle: t
 val part: ?label: label -> t -> t
   (** For the report style. *)
 
-val chapter: ?label: label -> t -> t
-val section: ?label: label -> t -> t
-val subsection: ?label: label -> t -> t
-val subsubsection: ?label: label -> t -> t
+val chapter: ?label: label -> ?short: t -> t -> t
+val section: ?label: label -> ?short: t -> t -> t
+val subsection: ?label: label -> ?short: t -> t -> t
+val subsubsection: ?label: label -> ?short: t -> t -> t
 val paragraph: t -> t
+
+val chapter': ?label: label -> ?short: t -> t -> t
+  (** Same as [chapter] but with no numbering. *)
+val section': ?label: label -> ?short: t -> t -> t
+  (** Same as [section] but with no numbering. *)
+val subsection': ?label: label -> ?short: t -> t -> t
+  (** Same as [subsection] but with no numbering. *)
+val subsubsection': ?label: label -> ?short: t -> t -> t
+  (** Same as [subsubsection] but with no numbering. *)
+
 val par: t
 val displaymath: t -> t
 val equation: ?label: label -> t -> t
@@ -429,8 +439,6 @@ val center: t -> t
 val flushleft: t -> t
 val flushright: t -> t
 
-val minipage: size -> t -> t
-
 val quote: t -> t
 val quotation: t -> t
 
@@ -443,6 +451,9 @@ val hspace: size -> t
 val addvspace: size -> t
   (** Similar to [vspace], but an [addvspace x] followed by an [addvspace y]
 will produce an [addvspace] of [max x y]. *)
+val ignorespaces: t
+  (** Tells LaTeX to ignore following spaces and new lines. Useful at the
+      end of a display environment, for instance. *)
 val smallskip: t (** A small [vspace]. *)
 val medskip: t (** A medium [vspace]. *)
 val bigskip: t (** A big [vspace]. *)
@@ -453,29 +464,74 @@ val phantom: t -> t
 val vphantom: t -> t (** Vertical-only phantom *)
 val hphantom: t -> t (** Horizontal-only phantom *)
 
-val parbox: size -> t -> t
+val rule_: ?lift:size -> size -> size -> t
+  (** [rule_ width height] draws a rule (i.e. a black box) of width [width]
+      and height [height] (for instance a horizontal or vertical line).
+      The optional argument [lift] moves the rule up if positive and down
+      if negative.
+      A special case is when [width] is null. In this case the rule,
+      called a strut, does not display, it only makes sure that the
+      surrounding box has at least its height. *)
+
+type valignment = [ `T | `C | `B ]
+  (** (`T)op, (`C)enter, (`B)ottom. *)
+
+val parbox: size -> ?valign:valignment -> t -> t
   (** A box in which new lines and paragraphs may be used. Useful to display
-      code listings, for instance. *)
+      code listings, for instance.
+      The [valign] optional argument controls the vertical alignment of
+      the box with respect to the surrounding text. *)
+val minipage: size -> ?valign:valignment -> t -> t
+  (** A box in which almost all command may be used. A more robust kind of
+      [parbox]. *)
 
 type halignment = [ `C | `L | `R | `S ]
   (** (`C)enter, flush (`L)eft, flush (`R)ight or (`S)pread. *)
 
-val makebox : size -> ?halign:halignment -> t -> t
+type xsize = [
+    size
+  | `Width of float
+  | `Height of float
+  | `Depth of float
+  | `Totalheight of float
+  ]
+(** Horizontal box commands ({!makebox}, {!framebox} and {!raisebox})
+    can use extra size information in their definition. These are computed
+    from their content:
+    [`Width] is the width of the content
+    [`Height] is the height above the baseline
+    [`Depth] is the height below the baseline
+    [`Totalheight] is the sum of [`Height] and [`Depth]
+ *)
+
+val makebox : xsize -> ?halign:halignment -> t -> t
   (** A box which only deals with horizontaly aligned material. *)
-val framebox : size -> ?halign:halignment -> t -> t
+val framebox : xsize -> ?halign:halignment -> t -> t
   (** Same as [makebox] but draws a frame around the box. *)
 
+val raisebox : shift:xsize -> ?fakeheight:xsize*xsize -> t -> t
+  (** [raisebox ~shift x] displays x vertically displaced by [shift].
+      If [~fakeheight] is not specified, then the line is built as if
+      [x] had not been moved.
+      If [~fakeheight:(h,d)] then the line building algorithm sees a box
+      which extends [h] above the baseline (height) and [d] below the
+      baseline (depth). *)
+
 
 type alignment = [ `L | `C | `R ]
 type array_column =  [ alignment | `Vert | `Sep of t]
 type array_line
 
-val array: array_column list -> array_line list -> t
+val array: ?valign:valignment -> array_column list -> array_line list -> t
 val array_line: ?sep: size -> ?layout:(int*[alignment|`I]) list -> t list -> array_line
   (** Extra alignment [`I] in layout means that the column inherits the alignment
         of the first corresponding column in the array layout.
         The integers in the layout correspond to over how many of the array's column
         will the cell will span.*)
+val array_command : t -> array_line
+  (** [array_command x] is a low level command. It gives [x] as an array line to Latex.
+         Meant to define alternative commands to draw horizontal lines in arrays.*)
+(*spiwack: todo: horizontal line commands, like [hline], [midrule], etc… *)
 
 (* Actually, I don't know what these do. *)
 val frontmatter: t
@@ -532,7 +588,11 @@ val breve: t -> t
 val tilde: t -> t
 val ddot: t -> t
 val widehat: t -> t
+  (** A wide [hat] which spreads over the whole argument. *)
 val widetilde: t -> t
+  (** A wide [tilde] which spreads over the whole argument. *)
+val overline: t -> t
+  (** A wide [bar] which spreads over the whole argument. *)
 
 (** {3 Greek Letters} *)
 
@@ -829,8 +889,10 @@ is translated to a number of line-skips *)
 
 (** The package ["stmaryrd"] is automatically added by these commands. *)
 
-val llbracket: t
-val rrbracket: t
+val llbracket: t (** [\[|] *)
+val rrbracket: t (** [|\]] *)
+val llparenthesis: t (** [(|] *)
+val rrparenthesis: t (** [|)] *)
 
 (** {3 Slide Document Class} *)
 
@@ -1150,13 +1212,21 @@ val addcontentsline: t -> t -> t -> t
 val pagestyle: t -> t
 val thispagestyle: t -> t
 
+val list_insert: 'a -> 'a list -> 'a list
+  (** Inserts an element between each elements of a list.
+
+      Examples:
+      - [list_insert 1 [] = []]
+      - [list_insert 1 [2] = [2]]
+      - [list_insert 1 [2; 3; 4] = [2; 1; 3; 1; 4]] *)
+
 (** {2 Printing} *)
 
 type env
 (* environment used to keep track of the content of variables between
    multiple applications of to_* functions *)
 
-val get_in_env: 'a variable -> env -> 'a
+val get_in_env: ?position:position -> 'a variable -> env -> 'a
 
 (** All printing functions take the expected mode as a parameter
 (default is text). The printed expression will be coerced if its mode differs. *)
